Unit FAT16_Lib;

interface
// global part

uses;

{--------------------------------------------------------------}
{ Const Declarations }
const

{$IDATA}
{--------------------------------------------------------------}
{ Type Declarations }

type
  tFileCmd       = (cmdSync, cmdGetState, cmdSetAddr, cmdRestart, cmdDiskFree,
  cmdDiskSize, cmdFileAttr, cmdEraseAll, cmdCreateDir, cmdEraseDir,
  cmdChangeDir, cmdPathExist, cmdCurDir, cmdFileExist, cmdFileSize,
  cmdFileRename, cmdFileCopy, cmdFileDelete, cmdListFiles,
  cmdFileAssign, cmdFileOpen, cmdCheckHandle, cmdFilePos, cmdFileSeek,
  cmdBlockRead, cmdBlockWrite, cmdFileClose, cmdFileCreate,
  cmdRandWrite, cmdEndOfFile, cmdFileSizeH, cmdRawAccess);

  tDiskState     = (dsNoDisk, dsWriteProt, dsDiskErr, dsDiskOk);

  tMediaState    = (msUnKnown, msDiskChanged, msInit);

  FAT16Lib_State = Record
                     Disk           : tDiskState;
                     Media          : tMediaState;
                     DiskErr        : Byte;
                     CommErr        : Byte;
                     FAT16Handle    : Byte;
                   end;

  Fat16_Ret      = Record
                     Cmd            : tFileCmd;
                     OK             : Char;
                   end;

{--------------------------------------------------------------}
{ Var Declarations }
var
  Ret            : Fat16_Ret;

{--------------------------------------------------------------}
{ functions }
  Procedure FAT16Lib_SetI2CAddr(I2C_Addr : Byte);
Function  FAT16Lib_GetI2CAddr : Byte;
Procedure FAT16Lib_Sync;
{ Function FAT16Lib_GetDiskState(VAR State : FAT16Lib_State) : Boolean;}
Function FAT16Lib_Init : Boolean;
Function FAT16Lib_GetDiskFree(var DiskFree : Longint) : Boolean;
Function FAT16Lib_GetDiskSize(var DiskSize : Longint) : Boolean;
Function FAT16Lib_EraseAll : Boolean;
Function FAT16Lib_CreateDIR(Path : String[12]; Name : String[12]; Time : Word; Date : Word) : boolean;
Function FAT16Lib_EraseDIR(Path : String[12]; Name : String[12]) : Boolean;
Function FAT16Lib_AssignFile(Path : String[12]; Name : String[12]; var FileHandle : Word) : Boolean;
Function FAT16Lib_CheckHandle(FHandle : Word) : Boolean;
Function FAT16Lib_ChangeDir(Path : String[12]) : Boolean;
Function FAT16Lib_GetDir(var Dir : String[12]) : Boolean;
Function FAT16Lib_PathExist(Path : String[12]) : Boolean;
Function FAT16Lib_FileExist(Path : String[12]; Name : String[12]; Attr : Byte) : Boolean;
Function FAT16Lib_Rename(Path : String[12]; Name : String[12]; NewName : String[12]) : Boolean;
Function FAT16Lib_Copy(Path : String[12]; Name : String[12]; NewPath : String[12]; NewName : String[12]) : Boolean;
Function FAT16Lib_FileDelete(Path : String[12]; Name : String[12]) : Boolean;
Function FAT16Lib_FileCreate(Path : String[12]; Name : String[12]; Attr : Byte; Time : Word; Date : Word; Size : Longint) : Boolean;
Function FAT16Lib_OpenWrite(FHandle : Word; Attr : Byte; Time : Word; Date : Word) : Boolean;
Function FAT16Lib_FileClose(FHandle : Word) : Boolean;
Function FAT16Lib_WriteBlock(FHandle : Word; Data : String[10]) : Boolean;
{Function FAT16Lib_GetFilelist(Path:String[21];Name:String[12];Attr:Byte;VAR Filelist : String[60]):Boolean;}
implementation
// local part

{--------------------------------------------------------------}
{ Type Declarations }
type

{--------------------------------------------------------------}
{ Const Declarations }
const
  NAK            : Char      = #21;
  ACK            : Char      = #6;
{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}
var

  TWI_Addr       : Byte;
  tttt           : String[8];
  Timer1         : SysTimer32;
  ccc            : String[12];

{--------------------------------------------------------------}
{ functions }

Procedure FAT16Lib_SetI2CAddr(I2C_Addr : Byte);
Begin
  TWI_Addr := I2C_Addr;
end;

Function  FAT16Lib_GetI2CAddr : Byte;
Begin
  Return(TWI_Addr);
end;

Procedure FAT16Lib_Sync;
Begin
  Repeat
  Until TWIOut(TWI_Addr, cmdSync);
end;


{Function FAT16Lib_GetDiskState(VAR State : FAT16Lib_State) : Boolean;
VAR
  Dummy          : Record
                     ret        : Byte;
                     LokState       : FAT16Lib_State;
                   end;
Begin
  Repeat
  Until TWIOut(TWI_Addr, cmdGetState);
    Repeat
  Until TWIInp(TWI_Addr, dummy);
  State := Dummy.LokState;
  If Dummy.Lastcmd = Byte(cmdGetState) then
    Return(True);
  else
    Return(False);
  end_If;
end;}

Function FAT16Lib_Init : Boolean;
Begin
  Repeat
  Until TWIOut(TWI_Addr, cmdRestart);
  SetSysTimer(Timer1, 200);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret)or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdRestart)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;



Function FAT16Lib_GetDiskFree(var DiskFree : longint) : Boolean;
var
  Test_Str       : String[8];
  Dummy          : Record
                     Ret            : Fat16_Ret;
                     Size           : array[0..7] of Char;
                   end;
Begin
  Repeat
  Until TWIOut(TWI_Addr, cmdDiskFree);
  SetSysTimer(Timer1, 200);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Dummy) or isSystimerZero(Timer1);
  If (Dummy.Ret.Cmd = cmdDiskFree)  and (Dummy.Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Test_Str := ARRtostr(Dummy.Size);
    Diskfree:= Hextoint(Test_Str);
    Return(True);
  else
    Return(FALSE);
  end_If;
end;

Function FAT16Lib_GetDiskSize(var DiskSize : longint) : Boolean;
var
  Test_Str       : String[8];
  Dummy          : Record
                     Ret            : Fat16_Ret;
                     Size           : array[0..7] of Char;
                   end;
Begin
  Repeat
  Until TWIOut(TWI_Addr, cmdDiskSize);
  SetSysTimer(Timer1, 200);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Dummy) or isSystimerZero(Timer1);
  If (Dummy.Ret.Cmd = cmdDiskSize)  and (Dummy.Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Test_Str := ARRtostr(Dummy.Size);
    DiskSize := Hextoint(Test_Str);
    Return(True);
  else
    Return(FALSE);
  end_If;
end;


Function FAT16Lib_EraseAll : Boolean;
var
  RetCmd         : Byte;
  OK_NotOK       : Char;
Begin

  Repeat
  Until TWIOut(TWI_Addr, cmdEraseAll);
  SetSysTimer(Timer1, 200);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret) or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdEraseAll)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;

Function FAT16Lib_CreateDIR(Path : String[12]; Name : String[12]; Time : Word; Date : Word) : boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..50] of Char;
                   end;
  gesamt_String [@part_string]: String[50];

Begin
  gesamt_String:= Path + #0 + Name + #0 + InttoHex(Time) + InttoHex(Date);
  Repeat
  Until TWIOut(TWI_Addr, cmdCreateDir, part_String.DataBytes);
  SetSysTimer(Timer1, 500);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret)or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdCreateDir) and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;


Function FAT16Lib_EraseDIR(Path : String[12]; Name : String[12]) : Boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..26] of Char;
                   end;
  gesamt_String[@part_string]: String[26];

Begin
  gesamt_String:= Path + #0 + Name + #0;
  Repeat
  Until TWIOut(TWI_Addr, cmdEraseDir, part_String.DataBytes);
  SetSysTimer(Timer1, 200);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret) or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdEraseDir)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;

Function FAT16Lib_ChangeDir(Path : String[12]) : Boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..13] of Char;
                   end;
  gesamt_String [@part_string]: String[13];

Begin
  gesamt_String:= Path + #0;
  Repeat
  Until TWIOut(TWI_Addr, cmdChangeDir, part_String.DataBytes);
  SetSysTimer(Timer1, 200);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret) or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdChangeDir)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;

Function FAT16Lib_PathExist(Path : String[12]) : Boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..13] of Char;
                   end;
  gesamt_String [@part_string]: String[13];

Begin
  gesamt_String:= Path + #0;
  Repeat
  Until TWIOut(TWI_Addr, cmdPathExist, part_String.DataBytes);
  SetSysTimer(Timer1, 200);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret) or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdPathExist)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;

Function FAT16Lib_FileExist(Path : String[12]; Name : String[12]; Attr : Byte) : Boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..30] of Char;
                   end;
  gesamt_String [@part_string]: String[30];

Begin
  gesamt_String:= Path + #0 + Name + #0 + BytetoHex(Attr);
  Repeat
  Until TWIOut(TWI_Addr, cmdFileExist, part_String.DataBytes);
  SetSysTimer(Timer1, 200);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret) or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdFileExist)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;

Function FAT16Lib_Rename(Path : String[12]; Name : String[12]; NewName : String[12]) : Boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..43] of Char;
                   end;
  gesamt_String [@part_string]: String[43];

Begin
  gesamt_String:= Path + #0 + Name + #0 + NewName + #0;
  Repeat
  Until TWIOut(TWI_Addr, cmdFileRename, part_String.DataBytes);
  SetSysTimer(Timer1, 200);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret) or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdFileRename)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;

Function FAT16Lib_Copy(Path : String[12]; Name : String[12]; NewPath : String[12]; NewName : String[12]) : Boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..55] of Char;
                   end;
  gesamt_String [@part_string]: String[55];

Begin
  gesamt_String:= Path + #0 + Name + #0 + newPath + #0 + NewName + #0;
  Repeat
  Until TWIOut(TWI_Addr, cmdFileCopy, part_String.DataBytes);
  SetSysTimer(Timer1, 200);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret) or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdFileCopy)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;


Function FAT16Lib_FileDelete(Path : String[12]; Name : String[12]) : Boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..30] of Char;
                   end;
  gesamt_String [@part_string]: String[30];

Begin
  gesamt_String:= Path + #0 + Name + #0;
  Repeat
  Until TWIOut(TWI_Addr, cmdFileDelete, part_String.DataBytes);
  SetSysTimer(Timer1, 255);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret) or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdFileDelete)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;

Function FAT16Lib_GetDir(var Dir : String[12]) : Boolean;
var
  Ret            : Fat16_Ret;
  PathChar       : Char;
  i              : Byte;

Begin
  Repeat
  Until TWIOut(TWI_Addr, cmdCurDir);
  SetSysTimer(Timer1, 200);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret) or isSystimerZero(Timer1);
  If isSystimerZero(Timer1) then Return (FALSE);
  endif;

  i := 1;
  SetSysTimer(Timer1, 500);
  Repeat
    Repeat
      mDelay(10);
    Until TWIInp(TWI_Addr, PathChar) or isSystimerZero(Timer1);
    If isSystimerZero(Timer1) then Return (FALSE);
    endif;
    
    If PathChar <> #0
    then
      Dir[i] := PathChar;
      INC(i);
    else
      SetLength(Dir,i-1);
    endif;
  Until (PathChar = #0);
  Return(True);
end;



Function FAT16Lib_FileClose(FHandle : Word) : Boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..5] of Char;
                   end;
  gesamt_String [@part_string]: String[5];

Begin
  gesamt_String:= InttoHex(FHandle);
  Repeat
  Until TWIOut(TWI_Addr, cmdFileClose, part_String.DataBytes);
  SetSysTimer(Timer1, 255);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret)or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdFileClose)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;


Function FAT16Lib_FileCreate(Path : String[12]; Name : String[12]; Attr : Byte; Time : Word; Date : Word; Size : Longint) : Boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..50] of Char;
                   end;
  gesamt_String [@part_string]: String[50];

Begin
  gesamt_String:= Path + #0 + Name + #0 + BytetoHex(Attr) + InttoHex(Time) + InttoHex(Date) + Longtohex(Size);
  Repeat
  Until TWIOut(TWI_Addr, cmdFileCreate, part_String.DataBytes);
  SetSysTimer(Timer1, 255);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret)or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdFileCreate)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;

Function FAT16Lib_AssignFile(Path : String[12]; Name : String[12]; var FileHandle : Word) : Boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..30] of Char;
                   end;
  gesamt_String [@part_string]: String[30];
  Test_Str       : String[4];
  Dummy          : Record
                     Ret            : Fat16_Ret;
                     Size           : array[0..3] of Char;
                   end;
Begin
  gesamt_String:= Path + #0 + Name + #0;
  Repeat
  Until TWIOut(TWI_Addr, cmdFileAssign, part_String.DataBytes);
  SetSysTimer(Timer1, 255);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Dummy)or isSystimerZero(Timer1);
  If (Dummy.Ret.Cmd = cmdFileAssign)  and (Dummy.Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Test_Str := ARRtostr(Dummy.Size);
    filehandle := Hextoint(Test_Str);
    Return(True);
  else
    Return(FALSE);
  end_If;

end;

Function FAT16Lib_CheckHandle(FHandle : Word) : Boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..5] of Char;
                   end;
  gesamt_String [@part_string]: String[5];

Begin
  gesamt_String:= InttoHex(FHandle);
  Repeat
  Until TWIOut(TWI_Addr, cmdCheckHandle, part_String.DataBytes);
  SetSysTimer(Timer1, 255);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret) or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdCheckHandle)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;

Function FAT16Lib_OpenWrite(FHandle : Word; Attr : Byte; Time : Word; Date : Word) : Boolean;
var
  Retu           : Record
                     Cmd            : tFileCmd;
                     Wr             : Char;
                     OK             : Char;
                   end;
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..25] of Char;
                   end;
  gesamt_String [@part_string]: String[25];

Begin
  gesamt_String:= 'W' + InttoHex(FHandle) + BytetoHex(Attr) + InttoHex(Time) + InttoHex(Date);
  Repeat
  Until TWIOut(TWI_Addr, cmdFileOpen,  part_String.DataBytes);
  SetSysTimer(Timer1, 255);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Retu) or isSystimerZero(Timer1);
  If (Retu.Cmd = cmdFileOpen) and (Retu.Wr = 'W')  and (Retu.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;

Function FAT16Lib_WriteBlock(FHandle : Word; Data : String[10]) : Boolean;
var
  part_String    : Record
                     LengthByte     : Byte;
                     DataBytes      : array [1..25] of Char;
                   end;
  gesamt_String [@part_string]: String[25];

Begin
  gesamt_String:= InttoHex(FHandle) + InttoHex(10) + Data;
  Repeat
  Until TWIOut(TWI_Addr, cmdBlockWrite, part_String.DataBytes);
  SetSysTimer(Timer1, 400);
  Repeat
    mDelay(10);
  Until TWIInp(TWI_Addr, Ret)or isSystimerZero(Timer1);
  If (Ret.Cmd = cmdBlockWrite)  and (Ret.OK = ACK) and not isSystimerZero(Timer1) then
    Return(True);
  else
    Return(FALSE);
  end_If;
end;

initialization
// at StartUp

// finalization          // optional
// at System_ShutDown
end FAT16_Lib.


